/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
/* 

/// Brad and Mike Adaptive Function: CORREL (Correllation)

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void Correl (const MSXDataInfoRec * Input1, const MSXDataInfoRec * Input2, 
			 const MSXDataInfoRec * Periods, MSXDataInfoRec * Result)
{
		// Only do the analysis on the UNION of the two arrays, the smallest set.
	int startIdx = MSXMax(Input1->iFirstValid, Input2->iFirstValid);
	int endIdx = MSXMin(Input1->iLastValid, Input2->iLastValid);

	int i; // the looping index for main array

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The number (+ or -) change between prev period and cur period.
	int n;

	// Set these as the default, change the firstValid later if we are "truncating" results
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	double x = 0.0;
	double y = 0.0;
	double sumX = 0.0;
	double sumX2 = 0.0;
	double sumXY = 0.0;
	double sumY = 0.0;
	double sumY2 = 0.0;

	curStartPt = startIdx;
	for (i = startIdx; i <= endIdx; i++)
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// the ACTUAL number of periods being calculated.
		n = i - curStartPt + 1;
		
		x = Input1->pfValue[i];
		y = Input2->pfValue[i];

		sumX  += ForceFloatRange(x);
		sumX2 += ForceFloatRange(x*x);
		sumXY += ForceFloatRange(x*y);
		sumY  += ForceFloatRange(y);
		sumY2 += ForceFloatRange(y*y);	

		changePeriods = curStartPt - prevStartPt;

		if (changePeriods > 0)
		{
			for (int j = 0; j < changePeriods; j++)
			{	
				x = Input1->pfValue[curStartPt - j - 1];
				y = Input2->pfValue[curStartPt - j - 1];

				sumX  -= ForceFloatRange(x);
				sumX2 -= ForceFloatRange(x*x);
				sumXY -= ForceFloatRange(x*y);
				sumY  -= ForceFloatRange(y);
				sumY2 -= ForceFloatRange(y*y);
			}
		}
		else if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				x = Input1->pfValue[prevStartPt + j - 1];
				y = Input2->pfValue[prevStartPt + j - 1];

				//sumX  = x;
				sumX  += ForceFloatRange(ForceFloatRange(x));
				sumX2 += ForceFloatRange(ForceFloatRange(x*x));
				sumXY += ForceFloatRange(ForceFloatRange(x*y));
				sumY  += ForceFloatRange(ForceFloatRange(y));
				sumY2 += ForceFloatRange(ForceFloatRange(y*y));
			}
		}

		if (n <= 0)
			Result->pfValue[i] = 0.0;
		else
		{
			x = ForceFloatRange((sumX2 - ((sumX*sumX) / n))  *  (sumY2 - ((sumY*sumY) / n)));
			
			if( !IsDoubleZeroOrNeg(x))
				Result->pfValue[i] = (float)ForceFloatRange( ( sumXY - ((sumX*sumY) / n)) / sqrt(x) );
			else
				Result->pfValue[i] = 0.0;
		}

	}
}	
	/*
	// Only do the analysis on the UNION of the two arrays, the smallest set.
	int startIdx = MSXMax(Input1->iFirstValid, Input2->iFirstValid);
	int endIdx = MSXMin(Input1->iLastValid, Input2->iLastValid);

	int i; // the looping index for main array

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The number (+ or -) change between prev period and cur period.

	// Set these as the default, change the firstValid later if we are "truncating" results
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	double x = 0.0;
	double y = 0.0;
	double sumX = 0.0;
	double sumX2 = 0.0;
	double sumXY = 0.0;
	double sumY = 0.0;
	double sumY2 = 0.0;

	curStartPt = startIdx;
	for (i = startIdx; i <= endIdx; i++)
	{
		
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		x = Input1->pfValue[i];
		y = Input2->pfValue[i];

		sumX  += ForceFloatRange(x);
		sumX2 += ForceFloatRange(x*x);
		sumXY += ForceFloatRange(x*y);
		sumY  += ForceFloatRange(y);
		sumY2 += ForceFloatRange(y*y);	
 
		int n = i - curStartPt + 1;

		changePeriods = curStartPt - prevStartPt;

		if (changePeriods > 0)
		{
			for (int j = 0; j < changePeriods; j++)
			{	
				x = Input1->pfValue[curStartPt - j - 1];
				y = Input2->pfValue[curStartPt - j - 1];

				sumX  -= ForceFloatRange(x);
				sumX2 -= ForceFloatRange(x*x);
				sumXY -= ForceFloatRange(x*y);
				sumY  -= ForceFloatRange(y);
				sumY2 -= ForceFloatRange(y*y);
			}
		}
		
		else if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				x = Input1->pfValue[prevStartPt + j - 1];
				y = Input2->pfValue[prevStartPt + j - 1];

				//sumX  = x;
				sumX  += ForceFloatRange(ForceFloatRange(x));
				sumX2 += ForceFloatRange(ForceFloatRange(x*x));
				sumXY += ForceFloatRange(ForceFloatRange(x*y));
				sumY  += ForceFloatRange(ForceFloatRange(y));
				sumY2 += ForceFloatRange(ForceFloatRange(y*y));

			}
		}
		
		if (n <= 0)
			Result->pfValue[i] = 0.0;
		else
		{
			x = ForceFloatRange((sumX2 - ((sumX*sumX) / n))  *  (sumY2 - ((sumY*sumY) / n)));
			
			if( !IsDoubleZeroOrNeg(x))
				Result->pfValue[i] = (float)ForceFloatRange( ( sumXY - ((sumX*sumY) / n)) / sqrt(x) );
			else
				Result->pfValue[i] = 0.0;
		}

	}

  */
