/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
/* 

/// Brad and Mike Adaptive Function:  LLV (Lowest Low Value)

	// This function uses a faster algorithm like the one in SMA_Fast
*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void LLV (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The change (+ or -) of the sliding back edge index of 
						// the window each time through the main loop.
	
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	int LowestIdx   =  -1; 
	float lowest = FLT_MAX; // make it the largest float so any other number is less than it
	float tmp;

	curStartPt = startIdx;
   
	for (i = startIdx; i <= endIdx; i++ )
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// see if we are adding or subtracting elements this time.
		changePeriods = curStartPt - prevStartPt;

		// Get the newest element at the front of the "window"
		tmp = Input->pfValue[i];


		// initialize to the previous low. 
		// there are a few case where the result array at i is not set after this time
		Result->pfValue[i] = lowest;

		// *** Improvement over TA_LIB method.
		// I check for the new element being the highest before anything else,
		// because if it is a new max, then we don't need to look back for a new one
		if( tmp <= lowest )
		{
			LowestIdx = i;
			lowest = tmp;

			Result->pfValue[i] = lowest; 
		}
		else
		{
			// We might have to look for a new minimum if we are throwing away elements.
			if (changePeriods > 0)
			{
				// But Only if the Highest Element was in the part that we threw away. (and was not the newest element)
				if( LowestIdx < curStartPt)
				{
					LowestIdx = curStartPt;
					lowest = Input->pfValue[LowestIdx];
        
					// loop through the entire current lookback and find a new max, worst case.
					for(j = curStartPt + 1; j <= i; j++)
					{
					   tmp = Input->pfValue[j];
					   if( tmp <= lowest )
					   {
						  LowestIdx = j;
						  lowest = tmp;
					   }
					}
					Result->pfValue[i] = lowest;
				}
			}
		}

		// If we are adding elements, we just loop through them all and see if each is a new minimum
		// (no matter if we found a new min on the front end or not)
		if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				tmp = Input->pfValue[prevStartPt + j - 1];
				if (tmp < lowest)
				{
					LowestIdx = prevStartPt + j - 1;
					lowest = tmp;
					Result->pfValue[i] = lowest;
				}
			}
		}  
	}
}
   
   
