/* 

/// Brad and Mike Function:  ROC (Rate of Change)

  // Just a simple function, not adaptive.


  // Below is an excerpt from TA_LIB from SourceForge:
*/
  /* The interpretation of the rate of change varies widely depending
    * which software and/or books you are refering to.
    *
    * The following is the table of Rate-Of-Change implemented in TA-LIB:
    *       MOM     = (price - prevPrice)         [Momentum]
    *       ROC     = ((price/prevPrice)-1)*100   [Rate of change]
    *       ROCP    = (price-prevPrice)/prevPrice [Rate of change Percentage]
    *       ROCR    = (price/prevPrice)           [Rate of change ratio]
    *       ROCR100 = (price/prevPrice)*100       [Rate of change ratio 100 Scale]
    *
    * Here are the equivalent function in other software:
    *       TA-Lib  |   Tradestation   |    Metastock         
    *       =================================================
    *       MOM     |   Momentum       |    ROC (Point)
    *       ROC     |   ROC            |    ROC (Percent)
    *       ROCP    |   PercentChange  |    -     
    *       ROCR    |   -              |    -
    *       ROCR100 |   -              |    MO
    *
    * The MOM function is the only one who is not normalized, and thus
    * should be avoided for comparing different time serie of prices.
    * 
    * ROC and ROCP are centered at zero and can have positive and negative
    * value. Here are some equivalence:
    *    ROC = ROCP/100 
    *        = ((price-prevPrice)/prevPrice)/100
    *        = ((price/prevPrice)-1)*100
    *
    * ROCR and ROCR100 are ratio respectively centered at 1 and 100 and are
    * always positive values.
    */




#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

// here we assume that the period can not be less than 1, and if it is, it is set to be 1.
// this may not be ideal behavior if we want to be able to look into the future (less than 1).


// The periods for the ROC are slightly different than most indicators.
// A 1 period MA would reference the "today" bar only, whereas a 1 period
// ROC would reference the "yesterday" bar (and also the "today" bar, of course)

void ROC (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, const int type, 
			MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int period;
	int curStartPt;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// First Element is undefined, but it is set to zero here for a seed
	Result->pfValue[startIdx] = 0.0;

	for (int i = startIdx; i <= endIdx; i++)
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		curStartPt = i - period;

		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// Percent ROC
		if (type == 0)
		{
			if (!IsDoubleZero(Input->pfValue[curStartPt]))
				Result->pfValue[i] = (float)ForceFloatRange(
								((Input->pfValue[i] / Input->pfValue[curStartPt]) - 1) * 100
								);
			else
				Result->pfValue[i] = 0.0;
		}
		// Absolute ROC						
		else
			Result->pfValue[i] = (float)ForceFloatRange(
									Input->pfValue[i] - Input->pfValue[curStartPt]
									);
	}	
}
