/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void RSI (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				 MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int startPt; // Set to be the starting point (index) in the array each time.
	int period;  // Set to be the IDEAL number of lookback indicies each time
	int n;  // Set to be the ACTUAL number of lookback indicies each time.
	
	double CU[2] = {0,0};
	double CD[2] = {0,0};

	double today;
	double yesterday;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// Main Loop. 
	for (i = startIdx; i <= endIdx; i++)
	{	
		period = int(ForceIntRange(Periods->pfValue[i]));

		// always use this convention, no such thing as negative or zero period
		if(period < 1) 
			period = 1;
		
		startPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (startPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i+1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			startPt = startIdx;
		}

		n = i - startPt + 1;

		//if (n == 1)
		//	Result->pfValue[i] = 0.0;

		CU[0] = 0.0;
		CD[0] = 0.0;

		
		//first time through, n = 1, so we don't go through LOOP
		if (n == 1)
		{
			CU[0] = 0.0;
			CD[0] = 0.0;
		}
		
		// We only do an average on the first time! (we really don't even have to average, since
		// the calculations are relative, so we avoid the uneccessary division.
		else if (n == 2)
		{
			for (j = n; j >= 2; j--)
			{
				today = Input->pfValue[j - 1 + startPt];
				yesterday = Input->pfValue[j - 2 + startPt];
				
				if ((today - yesterday) > 0)
					CU[0] = CU[0] + (today - yesterday);
				else
					CD[0] = CD[0] + (yesterday - today);
			}
		}
		else // on subsequent passes, we simply take today's change up/down, and then smooth with wilder's method
		{
			today = Input->pfValue[i];
			yesterday = Input->pfValue[i-1];
			
			if ((today - yesterday) > 0)
				CU[0] = CU[0] + (today - yesterday);
			else
				CD[0] = CD[0] + (yesterday - today);
		}

		// Apply Wilder's Smoothing
		CU[0] = ( CU[1] * (n-1) + CU[0] ) / n; 
		CD[0] = ( CD[1] * (n-1) + CD[0] ) / n; 


		if (!IsDoubleZero(CU[0] + CD[0]))
			Result->pfValue[i] = (float)ForceFloatRange( 100 * (CU[0]  / (CU[0] + CD[0])) );
		else
			Result->pfValue[i] = 0.0;

		CU[1] = CU[0];
		CD[1] = CD[0];
	}	
}




//  Brad Ulrich - 04-03-2005
// Code Below was an attempt at a "sliding window optimized" version of the indicator, 
// its was not immediately successful, so was left in favor of the simpler and slower method

/*
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The number (+ or -) change between prev period and cur period.

	// Set these as the default, change the firstValid later if we are "truncating" results
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	double TotalGain = 0.0;
	double TotalLoss = 0.0;

	// curBar needs a seed value
	//double curBar = 0.0;
	double curBar = ForceFloatRange(Input->pfValue[startIdx]);
	double prevBar = 0.0; 
	double diffBars = 0.0;

	double AvgLoss, AvgGain, prevAvgGain = 0.0, prevAvgLoss = 0.0;

	double prevResult = 0.0;

	// this variable is set to
	bool firstValid = TRUE;

	curStartPt = startIdx;

	for (i = startIdx; i <= endIdx; i++)
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array. only occurs on last bar.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;

			// we also want the seed updated here. we set it so that it is equal to the next value,
			// making the net gain and loss for the first bar equal to zero.
			curBar = ForceFloatRange(Input->pfValue[i+1]);
		

			curStartPt = startIdx;

			firstValid = TRUE;
			//continue; // break loop, go to next iteration, not like an averaging loop.
		}
		
		prevBar = curBar;
		curBar = ForceFloatRange(Input->pfValue[i]);
		diffBars = curBar - prevBar;

		if (diffBars < 0)
			TotalLoss =  ForceFloatRange(TotalLoss - diffBars);
		else
			TotalGain = ForceFloatRange(TotalGain + diffBars);

		changePeriods = curStartPt - prevStartPt;

		if (changePeriods > 0)
		{
			for (int j = 0; j < changePeriods; j++)
			{	
				prevBar = curBar;
				curBar = ForceFloatRange(Input->pfValue[curStartPt - j - 1]);
				diffBars = curBar - prevBar;

		if (diffBars < 0)
			TotalLoss =  ForceFloatRange(TotalLoss + diffBars);
		else
			TotalGain = ForceFloatRange(TotalGain - diffBars);
			}
		}
		
		
		if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				prevBar = curBar;
				curBar = ForceFloatRange(Input->pfValue[curStartPt - j - 1]);
				diffBars = ForceFloatRange(curBar - prevBar);

			if (diffBars < 0)
				TotalLoss =  ForceFloatRange(TotalLoss - diffBars);
			else
				TotalGain = ForceFloatRange(TotalGain + diffBars); 
			}
		}

  
		int n = i - curStartPt + 1;  // the number of ACTUAL periods calculated

		if (n == 0)
		{
			Result->pfValue[i] = 0.0;
			prevResult = 0.0;
			continue;
		}

		AvgGain = ForceFloatRange(ForceFloatRange(TotalGain) / (double) n );
		AvgLoss = ForceFloatRange(ForceFloatRange(TotalLoss) / (double) n );

		//AvgGain = ForceFloatRange( (prevAvgGain * (n-1) + AvgGain) / n );
		//AvgLoss = ForceFloatRange( (prevAvgLoss * (n-1) + AvgLoss) / n );
		
		// set these for next time through
		//prevAvgGain = AvgGain;
		//prevAvgLoss = AvgLoss;

		
		// if this is the first valid result, this is all we do.

		if (IsDoubleZero(AvgLoss))
			Result->pfValue[i] = 100;
		else
			Result->pfValue[i] = (float)ForceFloatRange( 
						( AvgGain / AvgLoss ) 
						);

		// but if it is not, we use the smoothing
		if (!firstValid)
		{
			// Smoothing using Wilder's approach
			prevResult = Result->pfValue[i] = (float)ForceFloatRange( 
							( prevResult * (n-1) + Result->pfValue[i] ) / n
							); 
			//prevResult = Result->pfValue[i];
		}
		else
			prevResult = Result->pfValue[i]; // don't forget to set seed for prevResult
 

		// COULD be set back to TRUE if we have another truncated period.
		firstValid = FALSE;
	}
}
*/