/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void RVGI (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result)
{
		// Stores a temporary Result from a Call to the EMA function,
	MSXDataInfoRec Temp;

	// Return an empty array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	MSXDataInfoRec ConstPeriods;

	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &ConstPeriods) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}

	for (int i = ConstPeriods.iFirstValid; i <= ConstPeriods.iLastValid; i++)
		ConstPeriods.pfValue[i] = 10.0;

	// Call the EMA function and then pass the output of first as input to second.
	// We can use the results array as a temp array.
	Stdev(MasterData, Input, &ConstPeriods, 1, &Temp);


	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int j; // the looping index for the periods array.

	int startPt; // Set to be the starting point (index) in the array each time.
	int period;  // Set to be the IDEAL number of lookback indicies each time
	int n;  // Set to be the ACTUAL number of lookback indicies each time.
	
	double CU[2] = {0,0};
	double CD[2] = {0,0};

	double today;
	double yesterday;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// Main Loop. 
	for (i = startIdx; i <= endIdx; i++)
	{	
		period = int(ForceIntRange(Periods->pfValue[i]));

		// always use this convention, no such thing as negative or zero period
		if(period < 1) 
			period = 1;
		
		startPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (startPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i+1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			startPt = startIdx;
		}

		n = i - startPt + 1;

		//if (n == 1)
		//	Result->pfValue[i] = 0.0;

		CU[0] = 0.0;
		CD[0] = 0.0;

		
		//first time through, n = 1, so we don't go through LOOP
		if (n == 1)
		{
			CU[0] = 0.0;
			CD[0] = 0.0;
		}
		
		// We only do an average on the first time! (we really don't even have to average, since
		// the calculations are relative, so we avoid the uneccessary division.
		else if (n == 2)
		{
			for (j = n; j >= 2; j--)
			{
				//today = Input->pfValue[j - 1 + startPt];
				//yesterday = Input->pfValue[j - 2 + startPt];
				today = Temp.pfValue[j - 1 + startPt];
				yesterday = Temp.pfValue[j - 2 + startPt];
				
				if ((today - yesterday) > 0)
					CU[0] = CU[0] + Temp.pfValue[i];
				else
					CD[0] = CD[0] + Temp.pfValue[i];
			}
		}
		else // on subsequent passes, we simply take today's change up/down, and then smooth with wilder's method
		{
			//today = Input->pfValue[i];
			//yesterday = Input->pfValue[i-1];
			today = Temp.pfValue[i];
			yesterday = Temp.pfValue[i-1];
			
			if ((today - yesterday) > 0)
				CU[0] = CU[0] + Temp.pfValue[i];
			else
				CD[0] = CD[0] + Temp.pfValue[i];
		}

		// Apply Wilder's Smoothing
		CU[0] = ( CU[1] * (n-1) + CU[0] ) / n; 
		CD[0] = ( CD[1] * (n-1) + CD[0] ) / n; 


		if (!IsDoubleZero(CU[0] + CD[0]))
			Result->pfValue[i] = (float)ForceFloatRange( 100 * (CU[0]  / (CU[0] + CD[0])) );
		else
			Result->pfValue[i] = 0.0;

		CU[1] = CU[0];
		CD[1] = CD[0];
	}	

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);
	MSX_ARRAY_DEALLOC(&ConstPeriods);
}