/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// The Code below is Adapted from the Metastock Developers Kit Manual

#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXStruc.h"

double MSX_TrueRange(double LT, double HT, double CY) 
{
	double greatest = HT - LT;

	double val2 = fabs(CY - HT);
	if( val2 > greatest )
		greatest = val2;

	double val3 = fabs( CY - LT  );
	if( val3 > greatest )
		greatest = val3;

	return greatest;
}

// Make sure this function should not return d1 > d2 in some cases !!!
double MSXMax(double d1, double d2)
{
	if (d1 >= d2)
		return d1;
	else
		return d2;

}


double MSXMin(double d1, double d2)
{
	if (d1 < d2)
		return d1;
	else
		return d2;
}

/*
 * The following 7 cases explain how the +DM and -DM are
    * calculated on one period:
    *
    * Case 1:                       Case 2:
    *    C|                        A|
    *     |                         | C|
    *     | +DM1 = (C-A)           B|  | +DM1 = 0
    *     | -DM1 = 0                   | -DM1 = (B-D)
    * A|  |                           D| 
    *  | D|                    
    * B|
    *
    * Case 3:                       Case 4:
    *    C|                           C|
    *     |                        A|  |
    *     | +DM1 = (C-A)            |  | +DM1 = 0
    *     | -DM1 = 0               B|  | -DM1 = (B-D)
    * A|  |                            | 
    *  |  |                           D|
    * B|  |
    *    D|
    * 
    * Case 5:                      Case 6:
    * A|                           A| C|
    *  | C| +DM1 = 0                |  |  +DM1 = 0
    *  |  | -DM1 = 0                |  |  -DM1 = 0
    *  | D|                         |  |
    * B|                           B| D|
    *
    *
    * Case 7:
    * 
    *    C|
    * A|  |
    *  |  | +DM=0
    * B|  | -DM=0
    *    D|
    *
    * In case 3 and 4, the rule is that the smallest delta between
    * (C-A) and (B-D) determine which of +DM or -DM is zero.
    *
    * In case 7, (C-A) and (B-D) are equal, so both +DM and -DM are
    * zero.
    *
    * The rules remain the same when A=B and C=D (when the highs
    * equal the lows).

  */

double MSX_DM_Plus(double A, double B, double C, double D) 
{
// case 2
	if (C < A && D < B)
		return 0;
// case 5
	if (C < A && D > B)
		return 0;
// case 1
	if (C > A && D > B)
		return C - A;
// case 3 and 4
	if (C > A && D < B)
	{
		if ((C - A) < (B - D))
			return 0;
		else
			return C - A;

	}
// case 7
	if ((C - A == 0) && (B - D == 0))
		return 0;

// case 6
	if (C == A && D == B)
		return 0;

	// should never get here
	return 0;
}


double MSX_DM_Minus(double A, double B, double C, double D) 
{
// case 5
	if (C < A && D > B)
		return 0;
// case 1
	if (C > A && D > B)
		return 0;
// case 2
	if (C < A && D < B)
		return B-D;
// case 3 and 4
	if (C > A && D < B)
	{
		if ((C - A) > (B - D))
			return 0;
		else
			return B - D;

	}
// case 7
	if ((C - A == 0) && (B - D == 0))
		return 0;

// case 6
	if (C == A && D == B)
		return 0;

	// should never get here
	return 0;
}